local cv_blending = CV_RegisterVar {
	name = "dvfx_blending",
	defaultvalue = "On",
	possiblevalue = CV_OnOff,
}

local on_drift_switch = {}
local on_drift_release = {}

local function onDriftSwitch(p, level)
	for _, fn in ipairs(on_drift_switch) do
		fn(p, level)
	end
end

local function onDriftRelease(p, level)
	for _, fn in ipairs(on_drift_release) do
		fn(p, level)
	end
end

local cv_haste
local function getDriftSparkValue(p)
    if cv_haste == nil then
        cv_haste = CV_FindVar("kmp_haste") or { value = 0 }
    end

    local driftval, driftcharge = K_GetKartDriftSparkValue(p), p.kartstuff[k_driftcharge]

    if cv_haste.value then
        local accel = max(0, (9 - p.kartspeed - p.kartweight/2))
		local target = (26*4 + p.kartspeed*2)*8

        local LDC = p.NAlastdriftcharge or 0

        if target < driftval then
            return target, LDC
        end
    end

    return driftval, driftcharge
end

local function getDriftVFXData(p)
	local driftvfx = p.driftvfx

	if driftvfx == nil then
		p.driftvfx = {}
		driftvfx = p.driftvfx
	end
	
	return driftvfx
end

local function getDriftSwitch(dsone, oldcharge, charge)
	local dstwo = dsone*2
	local dsthree = dstwo*2
	
	-- Could maybe use a cycle but this will do
	if oldcharge < dsone and charge >= dsone then
		return 1
	elseif oldcharge < dstwo and charge >= dstwo then
		return 2
	elseif oldcharge < dsthree and charge >= dsthree then
		return 3
	end
	
	return 0
end

local function getDriftLevel(dsone, charge)
	local dstwo = dsone*2
	local dsthree = dstwo*2

	if charge < dsone then
		return 0
	elseif charge < dstwo then
		return 1
	elseif charge < dsthree then
		return 2
	end
	
	return 3
end

addHook("MapLoad", function()
	for p in players.iterate do
		p.driftvfx = nil
	end
end)

addHook("PlayerThink", function(p)
	if not p.mo then return end
	if leveltime < 2 then return end -- Not sure why but sometimes it spawns a boostclip at mapload? maybe this will fix it
	
	local driftvfx = getDriftVFXData(p)

    local dsone, driftcharge = getDriftSparkValue(p)
    local dstwo = dsone*2
    local dsthree = dstwo*2

    if driftvfx.lastcharge == nil then
        driftvfx.lastcharge = driftcharge
    end
	
	local switch = getDriftSwitch(dsone, driftvfx.lastcharge, driftcharge)
	
	if switch ~= 0 then
		onDriftSwitch(p, switch)
	end

	-- If you go off ramp with a drift charge, release and press drift button again, you release drift boost
	-- but your driftcharge gets non zero. Tested on gamma, it seems it gets set to 32, so i'm assuming thats enough
    if driftcharge <= 32 and driftvfx.lastcharge > 0 and p.kartstuff[k_driftboost] > 0 then
		local level = getDriftLevel(dsone, driftvfx.lastcharge)
		
		if level ~= 0 then
			onDriftRelease(p, level)
		end
    end

    driftvfx.lastcharge = driftcharge
end)

rawset(_G, "DVFX_OnDriftSwitch", function(fn)
	table.insert(on_drift_switch, fn)
end)

rawset(_G, "DVFX_OnDriftRelease", function(fn)
	table.insert(on_drift_release, fn)
end)

rawset(_G, "DVFX_GetData", getDriftVFXData)

rawset(_G, "DVFX_GetDriftSparkValue", getDriftSparkValue)

rawset(_G, "DVFX_GetDriftChargeLevel", getDriftLevel)

rawset(_G, "DVFX_DoBlending", function() return cv_blending.value end)