freeslot("MT_DRIFTBOOST_CLIP", "S_DRIFTBOOST_CLIP1", "S_DRIFTBOOST_CLIP2", "SPR_DRCL", "sfx_cock1")

local ANIMTIME = TICRATE/2
local ANIMFRAMES = 8
local VERTTHRUST = 14*FRACUNIT
local TIMETOLIVE = 105

local cv_clips = CV_RegisterVar {
	name = "dvfx_clips",
	defaultvalue = "On",
	possiblevalue = {
		Off = 0,
		On = 1,
		LocalOnly = 2,
	},
}

local cv_shadow = CV_RegisterVar {
	name = "dvfx_clips_shadows",
	defaultvalue = "On",
	possiblevalue = CV_OnOff,
}

local cv_sounds = CV_RegisterVar {
	name = "dvfx_sounds",
	defaultvalue = "On",
	possiblevalue = CV_OnOff,
}

mobjinfo[MT_DRIFTBOOST_CLIP] = {
    spawnstate = S_DRIFTBOOST_CLIP1,
    flags = MF_NOCLIPTHING|MF_BOUNCE,
    spawnhealth = 1,
    radius = 6*FRACUNIT,
    height = 12*FRACUNIT,
}

states[S_DRIFTBOOST_CLIP1] = {
    sprite = SPR_DRCL,
    frame = A|FF_ANIMATE|FF_FULLBRIGHT,
    var1 = ANIMFRAMES-1,
    var2 = ANIMTIME/ANIMFRAMES,
    tics = ANIMTIME,
    nextstate = S_DRIFTBOOST_CLIP2,
}

states[S_DRIFTBOOST_CLIP2] = {
    sprite = SPR_DRCL,
    frame = I|FF_ANIMATE|FF_FULLBRIGHT,
    var1 = ANIMFRAMES-1,
    var2 = ANIMTIME/ANIMFRAMES,
    tics = ANIMTIME,
    nextstate = S_DRIFTBOOST_CLIP2,
}

-- Grr
local shadows_to_spawn = {}

-- Will spawn shadow later so it gets scaled properly before it is drawn
local function P_AskSpawnShadow(mo)
    table.insert(shadows_to_spawn, mo)
end

addHook("PreThinkFrame", function()
    while #shadows_to_spawn > 0 do
        local mo = table.remove(shadows_to_spawn)

        if mo.valid then
            P_SpawnShadowMobj(mo)
        end
    end
end)

local function P_IsPlayerLocal(p)
	for i = 0, 3 do
		if p == displayplayers[i] then return true end
	end

	return false
end

-- Returns the Z momentum of the object, accounting for slopes if the object is grounded
local function P_GetMobjZMovement(mo)
    local slope = mo.standingslope

    if not P_IsObjectOnGround(mo) then
        return mo.momz
    end

    if not slope then
        return 0
    end

    local angDiff = R_PointToAngle2(0, 0, mo.momx, mo.momy) - slope.xydirection
    local speed = FixedHypot(mo.momx, mo.momy)

    return P_ReturnThrustY(mo, slope.zangle, P_ReturnThrustX(mo, angDiff, speed))
end

local function K_SpawnDriftBoostClip(player)
	if not P_IsPlayerLocal(player) and cv_clips.value == 2 then
		return
	end

    local pmo = player.mo

    local z = pmo.z + FixedMul(pmo.height, pmo.scale)/2
    local clip = P_SpawnMobj(pmo.x, pmo.y, z, MT_DRIFTBOOST_CLIP)

	if cv_shadow.value then
		P_AskSpawnShadow(clip)
	end
	
	if player.kartstuff[k_invincibilitytimer] > 0 then
		clip.extravalue1 = 1
		clip.colorized = true
	else
		clip.color = player.skincolor
	end

    clip.momz = P_GetMobjZMovement(pmo)
    P_SetObjectMomZ(clip, VERTTHRUST, true) -- A little thrust upwards
	
	clip.destscale = pmo.scale

    P_InstaThrust(clip, pmo.angle, 115*player.speed/100)

    -- To use by other scripts?
    clip.target = pmo

    clip.fuse = TIMETOLIVE
end

local function K_SpawnDriftBoostClipBounce(mo)
    local radius = mo.radius>>FRACBITS
    local x, y, z = mo.x, mo.y, mo.z

    -- Sparks spawn on floor or ceiling if on ground, and in object center otherwise
    if not P_IsObjectOnGround(mo) then
        z = z + FixedMul(mo.height, mo.scale)/2
    elseif mo.eflags & MFE_VERTICALFLIP then
        z = z + FixedMul(mo.height, mo.scale)
    end

    for i = 1, 3 do
        local dx, dy = N_RandomRange(-radius, radius), N_RandomRange(-radius, radius)
        local spark = P_SpawnMobj(x + dx, y + dy, z, MT_SPARKLETRAIL)

        spark.color = SKINCOLOR_WHITE -- mo.color
        spark.colorized = true

        spark.momx = dx*FRACUNIT
        spark.momy = dy*FRACUNIT

        spark.fuse = TICRATE/2
    end
end

-- Recreating this because:
-- 1 - K_SpawnSparkleTrail calls synched rng
-- 2 - We need less of sparkles
local function K_MiniSparkleTrail(mo)
	local radius = mo.radius>>FRACBITS
	local height = mo.height>>FRACBITS

	local dx = N_RandomRange(-radius, radius)*FRACUNIT
	local dy = N_RandomRange(-radius, radius)*FRACUNIT
	local dz = N_RandomRange(0, height)*FRACUNIT

	local spark = P_SpawnMobj(mo.x + dx, mo.y + dy, mo.z + dz, MT_SPARKLETRAIL)

	spark.color = mo.color
	spark.colorized = true
	
	if DVFX_DoBlending() then
		spark.blendmode = AST_ADD
	end

	spark.momx = dx*FRACUNIT
	spark.momy = dy*FRACUNIT

	spark.fuse = 5
end

addHook("MobjThinker", function(mo)
	if not mo.valid then return end

    if P_CheckDeathPitCollide(mo) then
        P_RemoveMobj(mo)
        return
    end
	
	if mo.extravalue1 == 1 then
		mo.color = 1 + leveltime % (MAXSKINCOLORS-1)
		K_MiniSparkleTrail(mo)
	end

    -- "TINKTINKTINKTINKTINKTINK" (c) some random youtube comment about terraria "boss 3" music
    local tink = false

    local onground = P_IsObjectOnGround(mo)

    if mo.lastonground ~= nil and not mo.lastonground and onground then
        tink = mo.lastmomz ~= nil and abs(mo.lastmomz) > 2*FRACUNIT

        if tink then
            P_SetObjectMomZ(mo, -mo.lastmomz/2, true)
        end
    end

    -- Weird hack to detect wall bounces - MFE_JUSTBOUNCEDWALL only gets set to player mobjs -_-
    --
    -- Still behaves weird on slopes sadly
    --if not (onground or mo.lastonground) and mo.lastmomx ~= nil and (mo.lastmomx ~= mo.momx or mo.lastmomy ~= mo.momy) then
    --    tink = true
    --end

    if tink then
        K_SpawnDriftBoostClipBounce(mo)
        S_StartSoundAtVolume(mo, sfx_tink, 127)
    end

    mo.lastmomx = mo.momx
    mo.lastmomy = mo.momy
    mo.lastmomz = mo.momz
    mo.lastonground = onground
end, MT_DRIFTBOOST_CLIP)

DVFX_OnDriftRelease(function(p, level)
	local driftvfx = DVFX_GetData(p)

	if level == 3 then
		driftvfx.boostcliptimer = 120
		driftvfx.boostcliprainbow = true
		
		if cv_sounds.value then
			S_StartSound(p.mo, sfx_kc5b)
			S_StartSound(p.mo, sfx_s3kc4l)
		end
	elseif level == 2 then
		driftvfx.boostcliptimer = 48
		driftvfx.boostcliprainbow = false
		
		if cv_sounds.value then
			S_StartSound(p.mo, sfx_kc5b)
		end
	elseif level == 1 and p.kartstuff[k_sneakertimer] and booststack and booststack.running then
		-- Spawn 1 (one) clip from extending sneaker boost in tech
		driftvfx.boostcliptimer = 32
		driftvfx.boostcliprainbow = false
	end
end)

DVFX_OnDriftSwitch(function(p, level)
	if cv_sounds.value and level == 3 then
        S_StartSound(nil, sfx_cock1, p)
	end
end)

addHook("PlayerThink", function(p)
	if not p.mo then return end
	
	local driftvfx = DVFX_GetData(p)
	
	if p.deadtimer or (p.pflags & PF_TIMEOVER) then
		driftvfx.boostcliptimer = 0
	end

    driftvfx.boostcliptimer = max(($ or 0) - 1, 0)
	
	if not cv_clips.value then return end

    if driftvfx.boostcliptimer then
        if (driftvfx.boostcliprainbow and (driftvfx.boostcliptimer % 10 == 0))
			or (not driftvfx.boostcliprainbow and driftvfx.boostcliptimer % 16 == 0) then
			
			K_SpawnDriftBoostClip(p)

            if driftvfx.boostcliprainbow then S_StartSound(p.mo, sfx_s3k77) end
        end
    end
end)

local function stopClips(p)
	local driftvfx = DVFX_GetData(p)
	
	driftvfx.boostcliptimer = 0
	driftvfx.boostcliprainbow = false
	
	S_StopSoundByID(p.mo, sfx_s3kc4l)
end

addHook("PlayerSpin", stopClips)
addHook("PlayerSquish", stopClips)
addHook("PlayerExplode", stopClips)