freeslot(
	"SPR_DREL", "SPR_DRES",

	"S_DRIFT_ELECTRO",

	"MT_DRIFT_ELECTRO"
)

local cv_electro = CV_RegisterVar {
	name = "dvfx_electricity",
	defaultvalue = "On",
	possiblevalue = CV_OnOff,
}

local cv_air = CV_RegisterVar {
	name = "dvfx_electricity_air",
	defaultvalue = "On",
	possiblevalue = CV_OnOff,
}

local cv_level = CV_RegisterVar {
	name = "dvfx_electricity_level",
	defaultvalue = 2,
	possiblevalue = {
		MIN = 1,
		MAX = 3,
	}
}

mobjinfo[MT_DRIFT_ELECTRO] = {
	spawnstate = S_DRIFT_ELECTRO,
	radius = 48*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOCLIPTHING|MF_NOGRAVITY|MF_SCENERY|MF_DONTENCOREMAP
}

states[S_DRIFT_ELECTRO] = {
  sprite = SPR_DREL,
  frame = A|FF_PAPERSPRITE|FF_FULLBRIGHT,
  tics = 2, -- state duration
  action = nil,
  nextstate = S_INVISIBLE,
}

local SCALE_BIG = FRACUNIT+1 -- Just so it doesn't equal to SCALE_MID
local SCALE_MID = FRACUNIT
local SCALE_SMALL = 4*FRACUNIT/5

local function spawnOrMove(x, y, z, type, pmo, field)
	local obj = pmo[field]
	
	if not (obj and obj.valid) then
		obj = P_SpawnMobj(x, y, z, type)
		pmo[field] = obj
	else
		if obj.state == S_INVISIBLE then
			P_SetOrigin(obj, x, y, z)
		else
			P_MoveOrigin(obj, x, y, z)
		end
	end
	
	return obj
end

-- -1 - left
-- 1 - right
local function getVFXScale(player, dir)
	if player.kartstuff[k_drift]*player.cmd.driftturn > 0 then -- Inward drifts
		if player.kartstuff[k_drift]*dir > 0 then
			return SCALE_SMALL
		elseif player.kartstuff[k_drift]*dir < 0 then
			return SCALE_BIG
		end
	elseif player.kartstuff[k_drift]*player.cmd.driftturn < 0 then -- Outward drifts
		if player.kartstuff[k_drift]*dir > 0 then
			return SCALE_BIG
		elseif player.kartstuff[k_drift]*dir < 0 then
			return SCALE_SMALL
		end
	end
	
	return SCALE_MID
end

addHook("MobjThinker", function(pmo)
	if cv_electro.value == 0 then return end
	if not pmo.valid then return end
	if not pmo.player then return end
	if not (cv_air.value or P_IsObjectOnGround(pmo)) then return end
	
	local dsone, driftcharge = DVFX_GetDriftSparkValue(pmo.player)
	local level = DVFX_GetDriftChargeLevel(dsone, driftcharge)
	
	if level < cv_level.value then return end

    local scale = mapobjectscale

	local forwardangle = pmo.angle-(ANG1*11)*pmo.player.kartstuff[k_drift]
	local moveangle = R_PointToAngle2(0, 0, pmo.momx, pmo.momy)
	--local moVisualRadius = pmo.radius + (14*pmo.scale)
	local moVisualRadius = FixedMul(FixedDiv(5*pmo.radius/3, pmo.scale), mapobjectscale)
	local numFrames = 6
	local curFrame = (leveltime % numFrames)|FF_PAPERSPRITE|FF_FULLBRIGHT|((leveltime % 2)*FF_TRANS70)

	local eflags = 0
	local color
	local electro
	
	if level == 1 then
		color = SKINCOLOR_SAPPHIRE
	elseif level == 2 then
		color = SKINCOLOR_KETCHUP
	else
		color = (1 + (leveltime % (MAXSKINCOLORS-1)))
	end
	
	local z = pmo.z
	
	if pmo.eflags & MFE_VERTICALFLIP then
		z = pmo.z + pmo.height - mobjinfo[MT_DRIFT_ELECTRO].height
		eflags = MFE_VERTICALFLIP
	end
	
	for i = 1, 2 do
		local dir = (i == 1) and -1 or 1
		local name = (i == 1) and "driftelectricity_left" or "driftelectricity_right"
	
		local x = pmo.x + P_ReturnThrustX(pmo, forwardangle - dir*ANGLE_135, moVisualRadius)
		local y = pmo.y + P_ReturnThrustY(pmo, forwardangle - dir*ANGLE_135, moVisualRadius)

		local angle = moveangle + dir*ANGLE_22h/2
		
		local scalefactor = getVFXScale(pmo.player, dir)
		local scale = FixedMul(scalefactor, scale)

		-- Left
		electro = spawnOrMove(x, y, z, MT_DRIFT_ELECTRO, pmo, name)
		electro.color = color
		electro.angle = angle
		electro.destscale = scale
		electro.momx = pmo.momx
		electro.momy = pmo.momy
		electro.momz = pmo.momz
		P_SetScale(electro, scale)
		electro.state = S_DRIFT_ELECTRO
		electro.frame = curFrame
		electro.eflags = ($ & ~MFE_VERTICALFLIP)|eflags
		electro.target = pmo
		
		if DVFX_DoBlending() then
			electro.blendmode = AST_ADD
		else
			electro.blendmode = AST_TRANSLUCENT
		end
		
		if scalefactor == SCALE_BIG then
			electro.sprite = SPR_DRES
		end
	end
end, MT_PLAYER)
